#!/bin/csh -f

# This is the file we use to make the serialized grammars for the parser.
# If you are on the Stanford NLP machines, you can use it to remake the 
# serialized parsers (such as when there have been incompatible software
# changes).  Don't forget to klog first so you can access the AFS corpora.
# If you are not on the Stanford, then the script won't work for you as is,
# since it contains hard-coded paths to various treebanks.  But it may still
# be useful to inspect it to see what options we used to generate the various
# supplied grammars.

## Uncomment this bit to run it with older parser version
# setenv CLASSPATH /u/nlp/distrib/lexparser-2004-03-24/javanlp.jar:

set wsjptb=/afs/ir/data/linguistic-data/Treebank/3/parsed/mrg/wsj
set ctb4=/afs/ir/data/linguistic-data/Chinese-Treebank/4/data/bracketed
set negra=/afs/ir/data/linguistic-data/NEGRA/penn-format-train-dev-test
set host=`hostname | cut -d. -f1`

mv -f serializedParsers.log serializedParsers.bak
uptime > serializedParsers.log

# English PCFG binary and text grammars

( echo "Running englishPCFG (goodPCFG) on $host -server" ; time java -server -ms300m -mx1000m edu.stanford.nlp.parser.lexparser.LexicalizedParser -goodPCFG -deleteSplitters "VP^NP,VP^VP,VP^SINV,VP^SQ" -saveToSerializedFile englishPCFG.ser.gz -saveToTextFile englishPCFG.txt -maxLength 40 -train $wsjptb 200-2199 -testTreebank $wsjptb 2200-2219 ) >>& ./serializedParsers.log

# English noTagSplit no rule compaction PCFG text grammar
( echo "Running englishPCFG-noTagSplit-noCompact on $host -server" ; time java -server -ms300m -mx1000m edu.stanford.nlp.parser.lexparser.LexicalizedParser -goodPCFG -deleteSplitters "VP^NP,VP^VP,VP^SINV,VP^SQ" -noTagSplit -saveToTextFile englishPCFG-noTagSplit.txt -compactGrammar 0 -maxLength 40 -train $wsjptb 200-2199 -testTreebank $wsjptb 2200-2219 ) >>& ./serializedParsers.log

# English Factored binary

## Not yet clear that goodFactored is better than -ijcai03 -- not on dev set
# ( echo "Running englishFactored (goodFactored) on $host -server" ; time java -server -ms200m -mx1800m edu.stanford.nlp.parser.lexparser.LexicalizedParser -goodFactored -saveToSerializedFile englishFactored.ser.gz -saveToTextFile englishFactored.txt -maxLength 40 -train $wsjptb 200-2199 -testTreebank $wsjptb 2200-2219 ) >>& ./serializedParsers.log
# ( echo "Running englishFactored (ijcai03 correctTags) on $host -server" ; time java -server -ms200m -mx1800m edu.stanford.nlp.parser.lexparser.LexicalizedParser -ijcai03 -v -printStates -compactGrammar 0 -correctTags -saveToSerializedFile englishFactored.ser.gz -saveToTextFile englishFactored.txt -maxLength 40 -train $wsjptb 200-2199 -testTreebank $wsjptb 2200-2219 ) >>& ./serializedParsers.log
( echo "Running englishFactored (ijcai03 replication) on $host -server" ; time java -server -ms200m -mx1800m edu.stanford.nlp.parser.lexparser.LexicalizedParser -ijcai03 -v -printStates -compactGrammar 0 -saveToSerializedFile englishFactored.ser.gz -saveToTextFile englishFactored.txt -maxLength 40 -train $wsjptb 200-2199 -testTreebank $wsjptb 2200-2219 ) >>& ./serializedParsers.log

# Chinese PCFG binary

( echo "Running chinesePCFG on $host -server" ; time java -server -mx800m edu.stanford.nlp.parser.lexparser.LexicalizedParser -tLPP edu.stanford.nlp.parser.lexparser.ChineseTreebankParserParams -chinesePCFG -encoding GB18030 -saveToSerializedFile chinesePCFG.ser.gz -maxLength 40 -train $ctb4 026-270,301-499,600-999 -testTreebank $ctb4 001-025 ) >>& ./serializedParsers.log
# new train list (Galen and Huihsin): 026-270,301-499,555-589,597-1041
# newer train list (Galen and Huihsin): 026-270,301-499,600-999
# this is all Xinhua minus Stanford devel and Bikel test

# Chinese Factored binary

( echo "Running chineseFactored on $host -server" ; time java -server -ms200m -mx1800m edu.stanford.nlp.parser.lexparser.LexicalizedParser -tLPP edu.stanford.nlp.parser.lexparser.ChineseTreebankParserParams -acl03chinese -saveToSerializedFile chineseFactored.ser.gz -maxLength 40 -train $ctb4 026-270,301-499,600-999 -testTreebank $ctb4 001-025 ) >>& ./serializedParsers.log

# explicitly saying tLPP on command line is currently needed for file encoding
# this should probably be fixed
( echo "Running chineseFactored from serialized check on $host -server" ; time java -server -ms200m -mx1800m edu.stanford.nlp.parser.lexparser.LexicalizedParser -tLPP edu.stanford.nlp.parser.lexparser.ChineseTreebankParserParams -maxLength 40 -loadFromSerializedFile chineseFactored.ser.gz -treebank $ctb4 001-025 ) >>& ./serializedParsers.log
# this doesn't work.
# ( echo "Running chineseFactored from serialized 2nd check on $host -server" ; time java -server -ms200m -mx1800m edu.stanford.nlp.parser.lexparser.LexicalizedParser -maxLength 40 -loadFromSerializedFile chineseFactored.ser.gz -treebank $ctb4 001-025 ) >>& ./serializedParsers.log


# German Factored binary

( echo "Running germanFactored on $host -server" ; time java -server -ms200m -mx1800m edu.stanford.nlp.parser.lexparser.LexicalizedParser -tLPP edu.stanford.nlp.parser.lexparser.NegraPennTreebankParserParams -hMarkov 1 -vMarkov 2 -vSelSplitCutOff 300 -maxLength 40 -saveToSerializedFile germanFactored.ser.gz -train $negra 1 -testTreebank $negra 2 ) >>& ./serializedParsers.log

# German Dependency parser
# This requires normalizing the dependency output to strip boundary symbol.
# ( echo "Running germanDep on $host -server" ; time java -server -ms200m -mx1800m edu.stanford.nlp.parser.lexparser.LexicalizedParser -tLPP edu.stanford.nlp.parser.lexparser.NegraPennTreebankParserParams -dep -hMarkov 1 -maxLength 40 -saveToSerializedFile germanDep.ser.gz -train $negra 1 -testTreebank $negra 2 ) >>& ./serializedParsers.log


## English just to check parser code regression (not saved)

## Just for reference
( echo "Running englishPCFG (acl03pcfg) on $host -server" ; time java -server -ms200m -mx800m edu.stanford.nlp.parser.lexparser.LexicalizedParser -acl03pcfg -maxLength 40 -train $wsjptb 200-2199 -testTreebank $wsjptb 2200-2219 ) >>& ./serializedParsers.log

## See if same results from serialized parser
( echo "Running englishFactored (ijcai03 from serialized) on $host -server" ; time java -server -ms200m -mx1000m edu.stanford.nlp.parser.lexparser.LexicalizedParser -v -maxLength 40 -loadFromSerializedFile englishFactored.ser.gz -testTreebank $wsjptb 2200 2219 ) >>& ./serializedParsers.log
# ( echo "Running englishFactored (ijcai03 with nodeprune) on $host -server" ; time java -server -ms200m -mx1800m edu.stanford.nlp.parser.lexparser.LexicalizedParser -ijcai03 -v -compactGrammar 0 -nodePrune true -maxLength 40 -train $wsjptb 200-2199 -testTreebank $wsjptb 2200-2219 ) >>& ./serializedParsers.log

## See if same results from text grammar parser
( echo "Running englishFactored (ijcai03 from textGrammar) on $host -server" ; time java -server -ms200m -mx1000m edu.stanford.nlp.parser.lexparser.LexicalizedParser -v -maxLength 40 -loadFromTextFile englishFactored.txt -treebank $wsjptb 2200 2219 ) >>& ./serializedParsers.log


## English models done for PASCAL RTE 2005

# English RTE Factored binary

( echo "Running rteFactored (from treebank) on $host server" ; time java -server -ms200m -mx1800m edu.stanford.nlp.parser.lexparser.LexicalizedParser -ijcai03 -saveToSerializedFile rteFactored.ser.gz -maxLength 40 -train $wsjptb 100-2199,9000 -testTreebank $wsjptb 2200-2219 ) >>& ./serializedParsers.log

# English RTE PCFG binary 

( echo "Running rtePCFG (from treebank) on $host server" ; time java -server -ms300m -mx1000m edu.stanford.nlp.parser.lexparser.LexicalizedParser -goodPCFG -deleteSplitters "VP^NP,VP^VP,VP^SINV,VP^SQ" -saveToSerializedFile rtePCFG.ser.gz -maxLength 40 -train $wsjptb 100-2199,9000 -testTreebank $wsjptb 2200 2219 ) >>& ./serializedParsers.log

uptime >> serializedParsers.log

mv -f serializedParsersPerformance.current serializedParsersPerformance.last
echo -n "Parser run on " > serializedParsersPerformance.current
date >> serializedParsersPerformance.current
grep 'N: 253\|N: 393\|Done testing on treebank\|Running ' serializedParsers.log >> serializedParsersPerformance.current
echo >> serializedParsersPerformance.current
echo >> serializedParsersPerformance.current

cat serializedParsersPerformance.current >> serializedParsersPerformance.txt
