package edu.stanford.nlp.parser.lexparser;

import edu.stanford.nlp.ling.TaggedWord;

import java.util.Collection;

/**
 * A lexicon class for Chinese.  Extends the current Lexicon class,
 * overriding its score and train methods to include a
 * ChineseUnknownWordModel.
 *
 * @author Roger Levy
 */
public class ChineseLexicon extends BaseLexicon {
  private static final boolean useRandomWalk = false;
  public static boolean useCharBasedUnknownWordModel = false;
  // public static boolean useMaxentUnknownWordModel = false;
  public static boolean useGoodTuringUnknownWordModel = false;

  private ChineseUnknownWordModel unknown;
  // private ChineseMaxentLexicon cml;
  private static final int STEPS = 1;
  private RandomWalk probRandomWalk;

  public ChineseLexicon(Options.LexOptions op) {
    super(op);
  }

  /* trains on a collection of trees */
  public void train(Collection trees) {
    super.train(trees);
    if (useRandomWalk) {
      probRandomWalk = new RandomWalk(trees, STEPS);
    }
    // if (useMaxentUnknownWordModel) {
    //  cml = new ChineseMaxentLexicon();
    //  cml.trainUnknownWordModel(trees);
    // } else {
      unknown = new ChineseUnknownWordModel();
      if (useGoodTuringUnknownWordModel) {
        unknown.useGoodTuring();
      }
      unknown.train(trees);
    // }
  }

  public double score(IntTaggedWord iTW, int loc) {

    double c_W = seenCounter.getCount(iTW);

    boolean seen = (c_W > 0.0);

    if (seen) {
      if (useRandomWalk) {
        return scoreRandomWalk(iTW);
      } else {
        return super.score(iTW, loc);
      }
    } else {
      double score;
      // if (useMaxentUnknownWordModel) {
      //  score = cml.score(iTW, 0);
      // } else {
        score = unknown.score(iTW);
      // }
      //      System.err.println("Unknown word " + iTW.wordString() + " with tag " + iTW.tagString() + ".  score: " + score);
      return score;
    }
  }

  private double scoreRandomWalk(IntTaggedWord itw) {
    TaggedWord tw = itw.toTaggedWord();
    String word = tw.value();
    String tag = tw.tag();

    return probRandomWalk.score(tag, word);
  }

}
