package edu.stanford.nlp.parser.lexparser;

import edu.stanford.nlp.ling.Label;
import edu.stanford.nlp.ling.Sentence;
import edu.stanford.nlp.ling.Word;
import edu.stanford.nlp.trees.Tree;

import java.util.*;

/**
 * A word-segmentation scheme using the max-match algorithm
 *
 * @author Galen Andrew
 */
public class MaxMatchSegmenter implements WordSegmenter {

  private Set words = new HashSet();
  private int maxLength = 10;

  public void train(Collection trees) {
    for (Iterator iter = trees.iterator(); iter.hasNext();) {
      Tree tree = (Tree) iter.next();
      List labels = tree.yield();
      for (Iterator iter2 = labels.iterator(); iter2.hasNext();) {
        Label label = (Label) iter2.next();
        if (label.value().length() <= maxLength) {
          words.add(label.value());
        }
      }
    }
  }

  public Sentence segmentWords(String s) {
    List segmentedWords = new ArrayList();
    int length = s.length();
    int start = 0;
    while (start < length) {
      int end = Math.min(length, start + maxLength);
      while (end > start + 1) {
        String nextWord = s.substring(start, end);
        if (words.contains(nextWord)) {
          segmentedWords.add(new Word(nextWord));
          break;
        }
        end--;
      }
      if (end == start + 1) {
        // character does not start any word in our dictionary
        segmentedWords.add(new Word(new String(new char[]{s.charAt(start)})));
        start++;
      } else {
        start = end;
      }
    }

    return new Sentence(segmentedWords);
  }
}
