package edu.stanford.nlp.parser.lexparser;

import edu.stanford.nlp.trees.*;
import edu.stanford.nlp.trees.international.tuebadz.TueBaDZHeadFinder;
import edu.stanford.nlp.trees.international.tuebadz.TueBaDZLanguagePack;
import edu.stanford.nlp.ling.StringLabelFactory;

import java.io.Serializable;
import java.io.Reader;
import java.util.List;
import java.util.Arrays;

/** TreebankLangParserParams for the German Tuebingen corpus.
 *
 *  @author Roger Levy (rog@stanford.edu)
 */
public class TueBaDZParserParams extends AbstractTreebankParserParams {

  public TueBaDZParserParams() {
    super(new TueBaDZLanguagePack());
  }

  /** Returns the first sentence of TueBaDZ. */
  public List defaultTestSentence() {
    return Arrays.asList(new String[] { "Veruntreue" , "die" , "AWO" , "Spendengeld" , "?" });
  }

  public String[] sisterSplitters() {
    return new String[0];
  }

  public TreeTransformer collinizer() {
    return new TreeCollinizer(tlp);
  }

  public TreeTransformer collinizerEvalb() {
    return new TreeCollinizer(tlp);
  }

  public MemoryTreebank memoryTreebank() {
    return new MemoryTreebank(treeReaderFactory());
  }

  public DiskTreebank diskTreebank() {
    return new DiskTreebank(treeReaderFactory());
  }

  private static class TueBaDZTreeReaderFactory implements TreeReaderFactory, Serializable {

    boolean treeNormalizerInsertNPinPP = false;

    boolean treeNormalizerLeaveGF = false;

    public TreeReader newTreeReader(Reader in) {
      final TreeNormalizer tn = new TreeNormalizer();

      return new PennTreeReader(in, new LabeledScoredTreeFactory(new StringLabelFactory()), tn);
    }

  }

  public TreeReaderFactory treeReaderFactory() {
    return new TueBaDZTreeReaderFactory();
  }

  /** No options yet.*/
  public int setOptionFlag(String[] args, int i) {
    return i;
  }

  public void display() {
    System.out.println("This is a TueBaDZParserParams.");
  }

  /** returns a {@link TueBaDZHeadFinder}. */
  public HeadFinder headFinder() {
    return new TueBaDZHeadFinder();
  }


  /** A no-op right now. */
  public Tree transformTree(Tree t, Tree root) {
    return t;
  }

}
