package edu.stanford.nlp.trees;

import edu.stanford.nlp.ling.Label;

import java.io.Serializable;


/**
 * An individual dependency between a governor and a dependent.
 * The governor and dependent are represented as a Label.
 * For example, these can be a
 * Word or a WordTag.  If one wishes the dependencies to preserve positions
 * in a sentence, then each can be a LabeledConstituent.
 * Dependencies support an object naming the dependency type.  This may be
 * null.  Dependencies have factories.
 *
 * @author Christopher Manning
 */
public interface Dependency extends Serializable {

  /**
   * Describes the governor (regent/head) of the dependency relation.
   */
  public Label governor();

  /**
   * Describes the dependent (argument/modifier) of
   * the dependency relation.
   */
  public Label dependent();

  /**
   * Names the type of dependency (subject, instrument, ...).
   * This might be a String in the simplest case, but can provide for
   * arbitrary object types.
   */
  public Object name();

  /**
   * Are two dependencies equal if you ignore the dependency name.
   */
  public boolean equalIgnoringName(Object o);

  /**
   * Provide different printing options via a String keyword.
   * The main recognized option currently is "xml".  Otherwise the
   * default toString() is used.
   */
  public String toString(String format);

  /**
   * Provide a factory for this kind of dependency
   */
  public DependencyFactory dependencyFactory();

}
