package edu.stanford.nlp.trees.international.negra;

import edu.stanford.nlp.ling.StringLabel;
import edu.stanford.nlp.ling.LabelFactory;
import edu.stanford.nlp.ling.Label;

import java.util.HashMap;
import java.util.Map;
import java.util.Iterator;

/* An object for edge labels as implemented in the Negra treebank.
 * Negra labels need a CATEGORY, an EDGE LABEL, and a MORPHOLOGICAL
 * INFORMATION value.  There is a different object for lexicalized
 * Negra labels.
 *
 * @author Roger Levy
 */

public class NegraLabel extends StringLabel {

  public static final String FEATURE_SEP = "#";

  private String edge;

  public LabelFactory labelFactory() {
    return new NegraLabelFactory();
  }

  private static class NegraLabelFactory implements LabelFactory {

    public Label newLabel(String labelStr) {
      return new NegraLabel(labelStr);
    }

    /**
     * Options don't do anything.
     * @param labelStr
     * @param options
     * @return
     */
    public Label newLabel(String labelStr, int options) {
      return newLabel(labelStr);
    }

    /**
     * Nothing fancy now: just makes the argument the value of the label
     * @param encodedLabelStr
     * @return
     */
    public Label newLabelFromString(String encodedLabelStr) {
      return newLabel(encodedLabelStr);
    }

    /**
     * Iff oldLabel is a NegraLabel, copy it.
     * @param oldLabel
     * @return
     */
    public Label newLabel(Label oldLabel) {
      NegraLabel result;
      if(oldLabel instanceof NegraLabel) {
        NegraLabel l = (NegraLabel) oldLabel;
        result = new NegraLabel(l.value(),l.getEdge(), new HashMap());
        for(Iterator<Map.Entry> i = l.features.entrySet().iterator(); i.hasNext(); ) {
          Map.Entry e = (Map.Entry) i.next();
          result.features.put(e.getKey(),e.getValue());
        }
      }
      else
        result = new NegraLabel(oldLabel.value());
      return result;
    }
  }

  private Map features;

  public void setEdge(String edge) {
    this.edge = edge;
  }

  public String getEdge() {
    return edge;
  }

  private NegraLabel() {
  }

  public NegraLabel(String str) {
    this(str, new HashMap());
  }

  public NegraLabel(String str, Map features) {
    this(str, null, features);
  }

  public NegraLabel(String str, String edge, Map features) {
    super(str);
    this.edge = edge;
    this.features = features;
  }

  public void setFeatureValue(String feature, String value) {
    features.put(feature, value);
  }

  public String featureValue(String feature) {
    return (String) features.get(feature);
  }


  public String toString() {
    String str = value();
    if (edge != null) {
      str += "->" + getEdge();
    }
    str += "." + features.toString();
    return str;
  }


}
